@@ -16,7 +16,12 @@
                 names(formals(mat_sqrt)),names(formals(make_scaled_dist))))  
   dotnames <- setdiff(names(mc)[-1],names(formals(corrHLfit)))
   argcheck <- setdiff(dotnames,HLnames)
-  if (length(argcheck)) warning(paste("suspect argument(s) ",paste(argcheck, collapse=",")," in corrHLfit call."))
+  if (length(argcheck)) {
+    warning(paste("suspect argument(s) ",paste(argcheck, collapse=",")," in corrHLfit call."))
+    if ("offset" %in% argcheck) {
+      stop("the offset should be a formula term, not a distinct argument.")
+    }
+  }
   # 
   return(mc)
 }
