@@ -158,10 +158,15 @@
     APHLs_args$mu <- newmuetablob$mu
     #
     if (processed$p_v_obj=="p_v" && which_LevMar_step!="v") { ## new damping -> new weights -> new expensive computation to evaluate p_v
-      if (Trace) cat(stylefn(".")) # yellow in V_IN_B case
-      newsXaug <- do.call(processed$AUGI0_ZX$envir$method, # ie, def_AUGI0_ZX_sparsePrecision
-                        sXaug_arglist)
-      APHLs_args$sXaug <- newsXaug
+      if (processed$GLGLLM_const_w) {
+        newsXaug <- NULL
+        APHLs_args$sXaug <- sXaug
+      } else {
+        if (Trace) cat(stylefn(".")) # yellow in V_IN_B case
+        newsXaug <- do.call(processed$AUGI0_ZX$envir$method, # ie, def_AUGI0_ZX_sparsePrecision
+                            sXaug_arglist)
+        APHLs_args$sXaug <- newsXaug
+      }
     } else { ## sXaug_arglist will still be used after the loop !!!!!!!!!!!!!!!!!!!!
       APHLs_args$sXaug <- newsXaug <- NULL # to compute hlik, no expensive matrix computation.
     }
@@ -248,11 +253,13 @@
       breakcond <- "NaN_gain"
       break
     } 
-    div_gainratio <- (gainratio-prev_gainratio)*damping/dampingfactor
-    if (div_gainratio < -0.1) { ## F I X M E a bit arbirary but 1 may not make a big difference
-      breakcond <- "div_gain"
-      break  
-    } 
+    if (objname == "p_v") { # then the starting objective may be 'too high' and we need to handle that
+      div_gainratio <- (gainratio-prev_gainratio)*damping/dampingfactor
+      if (div_gainratio < -max(LevMarblob$dampDpD)/(1e05*damping)) { 
+        breakcond <- "div_gain" # used to switch from "V_IN_B" to "strict_v|b"
+        break 
+      } 
+    }
     if (gainratio > 0) { # gainratio may be always negative if initial ranefs better optimize logL than the correct solution does.
       ## cf Madsen-Nielsen-Tingleff again, and as in levmar library by Lourakis
       damping <- damping * max(1/3,1-(2*gainratio-1)^3) # gainratio->0 factor-> 2; gainratio->1 factor->1/3
@@ -299,13 +306,17 @@
     #   and sXaug may be needed to compute sscaled in .solve_v_h_IRLS()
     # For PQL fits newsXaug has not been needed in the damping loop but will be needed after exiting this fn
     #   (e.g., for its next call -> LevMarblob <- get_from_MME(sXaug=sXaug, which="LevMar_step", LMrhs=zInfo$scaled_grad, damping=damping))
-    if (Trace) { 
-      if (processed$p_v_obj=="p_v") { # v estimation within HL11
-        cat(stylefn_v("."))
-      } else  cat(stylefn(".")) # PQL/L, vb extimation
-    }
-    newsXaug <- do.call(processed$AUGI0_ZX$envir$method, sXaug_arglist)
-    APHLs_args$sXaug <- newsXaug
+    if (processed$GLGLLM_const_w) {
+      APHLs_args$sXaug <- newsXaug <- sXaug
+    } else {
+      if (Trace) { 
+        if (processed$p_v_obj=="p_v") { # v estimation within HL11
+          cat(stylefn_v("."))
+        } else  cat(stylefn(".")) # PQL/L, vb extimation
+      }
+      newsXaug <- do.call(processed$AUGI0_ZX$envir$method, sXaug_arglist)
+      APHLs_args$sXaug <- newsXaug
+    } 
     APHLs_args$which <- processed$p_v_obj # "p_v" # 
     newAPHLs <- do.call(".calc_APHLs_from_ZX", APHLs_args) 
   }
@@ -360,7 +371,7 @@
     }
   }
   RESU$muetablob <- muetablob <- .muetafn(eta=eta,BinomialDen=processed$BinomialDen,processed=processed) 
-  if ( ! LMMbool ) {
+  if ( ! processed$LLM_const_w && ! processed$GLGLLM_const_w) {
     RESU$w.resid <- .calc_w_resid(muetablob$GLMweights,phi_est)
     sXaug_arglist <- c(update_sXaug_constant_arglist, # contained H_global_scale but not longer so
                            list(w.ranef=wranefblob$w.ranef, 
@@ -575,10 +586,12 @@
     if ( ! is.null(for_intervals)) {
       currentDy <- (for_intervals$fitlik-oldlik)
       if (currentDy < -1e-4) .warn_intervalStep(oldlik,for_intervals)
+      if (LMMbool) etamo <-muetablob$sane_eta - off
       intervalBlob <- .intervalStep_spprec(old_v_h_beta=Vscaled_beta,
                                        sXaug=sXaug,zInfo=zInfo,
                                        for_intervals=for_intervals,
-                                       currentlik=oldlik,currentDy=currentDy)
+                                       currentlik=oldlik,currentDy=currentDy,
+                                       ZAL=ZAL, etamo=etamo)
       damped_WLS_blob <- NULL
       Vscaled_beta <- intervalBlob$v_h_beta
     } else if (LevenbergM) {
@@ -685,14 +698,26 @@
       dampings_env$v[[attr(damped_WLS_blob,"step")]] <- damped_WLS_blob$damping
       ## LevM PQL
       if (! is_HL1_1) {
-        if (damped_WLS_blob$lik < oldAPHLs$hlik) { ## if LevM step failed to find a damping that increases the hlik :
+        if (damped_WLS_blob$lik < oldAPHLs$hlik) { ## if LevM step failed to find a damping that increases the hlik
           # Tis should occur only bc of (1) numerically challenging conditions e.g mu close to bounds; or (2) optimum has been 
           # found and floating point innacurracies matter.
           damped_WLS_blob <- NULL
+          beta_cov_info <- .calc_beta_cov_info_spprec(X.pv = sXaug$AUGI0_ZX$X.pv,envir = sXaug$BLOB,w.resid = w.resid)
+          if ((current_kappa <- kappa(tcrossprod(beta_cov_info$tcrossfac_beta_v_cov)))>1e08) {
+            processed$envir$PQLdivinfo$high_kappa$ranFixes <- c(processed$envir$PQLdivinfo$high_kappa$ranFixes,
+                                                                list(processed$envir$ranFix))
+          } else processed$envir$PQLdivinfo$unknown$ranFixes  <- c(processed$envir$PQLdivinfo$unknown$ranFixes,
+                                                                   list(processed$envir$ranFix))
           dVscaled_beta <- get_from_MME(sXaug,szAug=zInfo) ################### FIT
           Vscaled_beta <- list(v_h=Vscaled_beta$v_h+dVscaled_beta$dv_h,
                                beta_eta=Vscaled_beta$beta_eta+dVscaled_beta$dbeta_eta)
-          LevenbergM <- FALSE ## D O N O T set it to TRUE again !
+          if (TRUE) { # not clear what is best here
+            break
+          } else {
+            LevenbergM <- FALSE ## desperate move... 
+            # for (st in names_keep) assign(st,keep_init[[st]])
+            # Vscaled_beta <- c(v_h/ZAL_scaling ,beta_eta) ## RHS from assign(st,keep_init[[st]])
+          }
         } 
       }
     } else { ## IRLS: always accept new v_h_beta
@@ -715,9 +740,9 @@
                                          wranefblob, Trace=trace, stylefn)
       for (st in names(WLS_blob)) assign(st,WLS_blob[[st]]) 
     } else {
-      for (st in c("Vscaled_beta","wranefblob","v_h","u_h","muetablob",
-                   "w.resid", ## !important! cf test-adjacency-corrMatrix.R
-                   "sXaug")) assign(st,damped_WLS_blob[[st]])
+      for (st in intersect(names(damped_WLS_blob), # sXaug (and the weights) need not be present if(processed$GLGLLM_const_w) 
+                           c("w.resid", ## !important! cf test-adjacency-corrMatrix.R
+                             "sXaug","Vscaled_beta","wranefblob","v_h","u_h","muetablob"))) assign(st,damped_WLS_blob[[st]])
       if ( ! GLMMbool ) {
         # Xscal <- damped_WLS_blob$Xscal # does not exist; and presumably not needed
         ZAL_scaling <- damped_WLS_blob$ZAL_scaling ## hmmm but this is the TAGged 1, isn't it ? FIXME
@@ -735,7 +760,9 @@
           cat(crayon::red("!"))
         } else if ( ! identical(processed$warned_maxit_mean, TRUE)) {
           processed$warned_maxit_mean <- TRUE
-          message("Iterative algorithm converges slowly. See help('convergence') for suggestions.")
+          if (!is.null(for_intervals)) {
+            message("Iterative algorithm converges slowly.")
+          } else message("Iterative algorithm converges slowly. See help('convergence') for suggestions.")
         }
       }
       break
@@ -808,7 +835,7 @@
   return(RESU)
 } 
 
-.intervalStep_spprec <- function(old_v_h_beta,sXaug,zInfo,currentlik,for_intervals,currentDy) { 
+.intervalStep_spprec <- function(old_v_h_beta,sXaug,zInfo,currentlik,for_intervals,currentDy,ZAL, etamo) { 
   #print((processed$intervalInfo$fitlik-currentlik)/(control.HLfit$intervalInfo$MLparm-old_betaV[parmcol]))
   ## voir code avant 18/10/2014 pour une implem rustique de VenzonM pour debugage  
   ## somewhat more robust algo (FR->FR: still improvable ?), updates according to a quadratic form of lik near max
@@ -824,6 +851,7 @@
     currentDx <- (old_v_h_beta_vec[parmcol_ZX]-for_intervals$MLparm)
     targetDy <- (for_intervals$fitlik-for_intervals$targetlik)
     Dx <- currentDx*sqrt(targetDy/currentDy)
+    #cat(currentDx," ",targetDy," ",currentDy," ",Dx,"\n")
     ## pb is if Dx=0 , Dx'=0... and Dx=0 can occur while p_v is still far from the target, because other params have not converged.
     ## (fixme) patch:
     if (currentDy<targetDy) { ## we are close to the ML: we extrapolate a bit more confidently
@@ -832,10 +860,19 @@
     Dx <- sign(currentDx)*max(abs(Dx),min_abs_Dx)
     v_h_beta_vec[parmcol_ZX] <- for_intervals$MLparm + Dx 
   }
-  ## szAug changes:
+  ## gradient changes: (the get_from_MME() call only uses m_grad_obj to get rhs)
+  # scratch code that led o this is in devel_confint_spprec.R
   parmcol_X <- for_intervals$parmcol_X
-  zInfo$z1 <- zInfo$z1 - sXaug$AUGI0_ZX$X.pv[,parmcol_X]*v_h_beta_vec[parmcol_ZX] 
-  zInfo$m_grad_obj <- zInfo$m_grad_obj[-parmcol_ZX]
+  off_newparm <- sXaug$AUGI0_ZX$X.pv[,parmcol_X]*v_h_beta_vec[parmcol_ZX] # function of NEW tentative value of MLparm
+  # so it does not give the difference between etamo and i_etamo, which depends on OLD value of MLparm
+  oovb <- old_v_h_beta_vec[-(parmcol_ZX)]
+  seq_n_u_h <- seq_along(old_v_h_beta$v_h)
+  i_etamo <- drop(ZAL %*% oovb[seq_n_u_h] +
+                   sXaug$AUGI0_ZX$X.pv[,-(parmcol_X),drop=FALSE] %*%  oovb[-seq_n_u_h]) # 
+  #
+  rhs <- attr(sXaug, "w.resid")*(etamo-off_newparm- i_etamo)
+  zInfo$m_grad_obj <- zInfo$m_grad_obj[-parmcol_ZX]+
+    c(drop(crossprod(ZAL,rhs)),drop(crossprod(sXaug$AUGI0_ZX$X.pv[,-parmcol_X,drop=FALSE],rhs))) # tjrs zInfoo$m_grad_obj
   ## sXaug changes: 
   int_AUGI0_ZX <- as.list(sXaug$AUGI0_ZX) ## as.list() local copy avoids global modifs of original sXaug$AUGI0_ZX envir
   int_AUGI0_ZX$X.pv <- int_AUGI0_ZX$X.pv[,-(parmcol_X),drop=FALSE]
@@ -844,6 +881,10 @@
   #  but less explicit than the following: 
   int_sXaug <- sXaug ## Assuming sXaug not being an envir, then the following does not affect sXaug.
   int_sXaug$AUGI0_ZX <- int_AUGI0_ZX ## Replaces an envir by a list i nthe local copy; 
+  # ?__F I X M E__? it's unsafe as $BLOB is left unchanged (and still belong to the original object) while it might contain info reused to solve the system:
+  # Theis attr(.,"pforpv") is used to determined whether there are beta coeffs to compute in .AUGI0_ZX_sparsePrecision()
+  # int_sXaug$BLOB <- list2env(list(), parent=environment(.AUGI0_ZX_sparsePrecision))
+  attr(int_sXaug,"pforpv") <- attr(int_sXaug,"pforpv") - length(parmcol_X) # a priori  -1
   v_h_beta_vec[-(parmcol_ZX)] <- old_v_h_beta_vec[-(parmcol_ZX)]+ unlist(get_from_MME(int_sXaug,szAug=zInfo)) 
   return(list(v_h_beta=relist(v_h_beta_vec,old_v_h_beta))) 
 }
